@@ -1,7 +1,7 @@
 /* gxmessage - an xmessage clone using GTK2
  *
- * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
- * Timothy Richard Musson
+ * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
+ * 2012 Timothy Richard Musson
  *
  * Email: Tim Musson <trmusson@gmail.com>
  * WWW:   http://homepages.ihug.co.nz/~trmusson/programs.html#gxmessage
@@ -32,9 +32,12 @@
 
 /* Details for Copyright and bug report messages: */
 #define AUTHOR  "Timothy Richard Musson"
-#define YEAR    "2009"
+#define YEAR    "2012"
 #define MAILTO  "<trmusson@gmail.com>"
 
+/* If the following file exists, the -noescape option will be allowed to work: */
+#define ALLOW_NOESCAPE PACKAGE_DATA_DIR \
+            G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "allow_noescape"
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -73,10 +76,13 @@ struct {
 	gint              timeout;
 	guint             timeout_id;
 	gboolean          do_iconify;
+	gboolean          do_ontop;
 	gboolean          do_print;
 	gboolean          do_buttons;
 	gboolean          do_borderless;
+	gboolean          do_sticky;
 	gboolean          do_focus;
+	gboolean          allow_escape;
 	GtkWrapMode       wrap_mode;
 	GtkWindowPosition window_position;
 	GtkWidget         *entry_widget;
@@ -117,6 +123,7 @@ static struct Option option[] = {
 	{2, TRUE,  "bd"},
 	{2, TRUE,  "bw"},
 	{1, FALSE, "iconic"},
+	{1, FALSE, "ontop"},
 	{2, TRUE,  "xrm"},
 	{2, FALSE, "rv"},
 	{2, FALSE, "reverse"},
@@ -126,9 +133,11 @@ static struct Option option[] = {
 	{2, TRUE,  "name"},
 	{2, TRUE,  "display"},
 	{2, FALSE, "borderless"},
+	{2, FALSE, "sticky"},
 	{1, FALSE, "wrap"},
 	{3, TRUE,  "encoding"},
 	{3, FALSE, "nofocus"},
+	{3, FALSE, "noescape"},
 	{6, TRUE,  "entrytext"},
 	{3, FALSE, "entry"},
 	{1, FALSE, "?"},
@@ -155,6 +164,7 @@ enum {
 	OPT_BD,
 	OPT_BW,
 	OPT_ICONIC,
+	OPT_ONTOP,
 	OPT_XRM,
 	OPT_RV,
 	OPT_REVERSE,
@@ -164,9 +174,11 @@ enum {
 	OPT_NAME,
 	OPT_DISPLAY,
 	OPT_BORDERLESS,
+	OPT_STICKY,
 	OPT_WRAP,
 	OPT_ENCODING,
 	OPT_FOCUS,
+	OPT_NOESCAPE,
 	OPT_ENTRYTEXT,
 	OPT_ENTRY,
 	OPT_HELP_Q,
@@ -345,7 +357,7 @@ cb_window_destroy (GtkWidget *widget, GdkEvent *event, gpointer data)
 gboolean
 cb_key_press (GtkWidget *w, GdkEventKey *event, gpointer data)
 {
-	if (event->keyval == GDK_Escape) {
+	if (gx.allow_escape && (event->keyval == GDK_Escape)) {
 		gtk_main_quit ();
 	}
 	return FALSE;
@@ -467,6 +479,14 @@ window_create (void)
 		gtk_window_iconify (GTK_WINDOW(window));
 	}
 
+	if (gx.do_sticky) {
+		gtk_window_stick (GTK_WINDOW(window));
+	}
+
+	if (gx.do_ontop) {
+		gtk_window_set_keep_above (GTK_WINDOW(window), TRUE);
+	}
+
 	if (gx.do_borderless) {
 		gtk_window_set_decorated (GTK_WINDOW(window), FALSE);
 	}
@@ -743,7 +763,7 @@ usage (void)
 	g_print(_("  -buttons BUTTON_LIST   List of \"LABEL:EXIT_CODE\", comma separated\n"));
 	g_print(_("  -default LABEL         Give keyboard focus to the specified button\n"));
 	g_print(_("  -print                 Send the selected button's LABEL to stdout\n"));
-	g_print(_("  -center                Open the window in the centre of the screen\n"));
+	g_print(_("  -center                Open the window in the center of the screen\n"));
 	g_print(_("  -nearmouse             Open the window near the mouse pointer\n"));
 	g_print(_("  -timeout SECONDS       Exit with code 0 after SECONDS seconds\n"));
 	g_print(_("  -display DISPLAY       X display to use\n"));
@@ -757,7 +777,10 @@ usage (void)
 	g_print("\n");
 	g_print(_("Additional %s options:\n"), PACKAGE);
 	g_print(_("  -borderless            Open the window without border decoration\n"));
+	g_print(_("  -sticky                Make the window stick to all desktops\n"));
+	g_print(_("  -ontop                 Keep window on top\n"));
 	g_print(_("  -nofocus               Don't focus the window when it opens\n"));
+	g_print(_("  -noescape              Don't allow pressing ESC to close the window\n"));
 	g_print(_("  -encoding CHARSET      Expect CHARSET as the message encoding\n"));
 	g_print(_("  -entry                 Prompt for text to be sent to stdout\n"));
 	g_print(_("  -entrytext TEXT        Same as -entry, but with TEXT as default text\n"));
@@ -809,7 +832,9 @@ main (gint argc, gchar *argv[])
 	ok = my_gtk_init (argc, argv);
 
 	gx.exit_code       = 1;
+	gx.do_ontop        = FALSE;
 	gx.do_focus        = TRUE;
+	gx.allow_escape    = TRUE;
 	gx.wrap_mode       = GTK_WRAP_NONE;
 	gx.window_position = GTK_WIN_POS_NONE;
 
@@ -921,6 +946,12 @@ main (gint argc, gchar *argv[])
 		case OPT_ICONIC:
 			gx.do_iconify = TRUE;
 			break;
+		case OPT_ONTOP:
+			gx.do_ontop = TRUE;
+			break;
+		case OPT_STICKY:
+			gx.do_sticky = TRUE;
+			break;
 		case OPT_BORDERLESS:
 			gx.do_borderless = TRUE;
 			break;
@@ -933,6 +964,11 @@ main (gint argc, gchar *argv[])
 		case OPT_FOCUS:
 			gx.do_focus = FALSE;
 			break;
+		case OPT_NOESCAPE:
+            if (g_file_test (ALLOW_NOESCAPE, G_FILE_TEST_EXISTS)) {
+    			gx.allow_escape = FALSE;
+            }
+			break;
 		case OPT_IS_MISSING_ARG:
 			/* in this case, xmessage treats the "option" as normal text */
 		case OPT_IS_UNKNOWN:
